#include <Evas.h>


#define E_SMART_OBJ_GET(smart, o, type) \
  { \
    char *_e_smart_str; \
    \
    if (!o) return; \
    \
    smart = evas_object_smart_data_get(o); \
    if (!smart) return; \
    \
    _e_smart_str = (char *)evas_object_type_get(o); \
    if (!_e_smart_str) return; \
    \
    if (strcmp(_e_smart_str, type)) return; \
  }

#define E_SMART_OBJ_GET_RETURN(smart, o, type, ret) \
  { \
    char *_e_smart_str; \
    \
    if (!o) return ret; \
    \
    smart = evas_object_smart_data_get(o); \
    if (!smart) return ret; \
    \
    _e_smart_str = (char *)evas_object_type_get(o); \
    if (!_e_smart_str) return ret; \
    \
    if (strcmp(_e_smart_str, type)) return ret; \
  }

#define E_OBJ_NAME "cervoise"


typedef struct _Smart_Cervoise Smart_Cervoise;

struct _Smart_Cervoise
{
  Evas_Object *bg; /* background */
  Evas_Object *o;  /* centered object */
};


static void _smart_init (void);
static void _smart_add (Evas_Object *obj);
static void _smart_del (Evas_Object *obj);
static void _smart_move (Evas_Object *obj, Evas_Coord x, Evas_Coord y);
static void _smart_resize (Evas_Object *obj, Evas_Coord w, Evas_Coord h);
static void _smart_show (Evas_Object *obj);
static void _smart_hide (Evas_Object *obj);
static void _smart_clip_set (Evas_Object *obj, Evas_Object *clip);
static void _smart_clip_unset (Evas_Object *obj);


/**********************************/
/* Globals for the Object */
/**********************************/
static Evas_Smart  *smart = NULL;


/*******************************/
/* Externally accessible calls */
/*******************************/

Evas_Object *
cervoise_add (Evas *evas)
{
  _smart_init();

  return evas_object_smart_add(evas, smart);
}


/*******************************************/
/* Internal smart object required routines */
/*******************************************/
static void
_smart_init (void)
{
  if (smart) return;
  static const Evas_Smart_Class sc =
    {
      E_OBJ_NAME,
      EVAS_SMART_CLASS_VERSION,
      _smart_add,
      _smart_del,
      _smart_move,
      _smart_resize,
      _smart_show,
      _smart_hide,
      NULL,
      _smart_clip_set,
      _smart_clip_unset,
      NULL,
      NULL,
      NULL,
      NULL
    };
  smart = evas_smart_class_new(&sc);
}

static void
_smart_add (Evas_Object *obj)
{
  Smart_Cervoise *sc;
  Evas_Object *o;
  int width;
  int height;
  int margin;

  sc = calloc (1, sizeof (Smart_Cervoise));
  if (!sc) return;

  o = evas_object_rectangle_add(evas_object_evas_get (obj));
  evas_object_move(o, 0, 0);
  evas_object_resize(o, 200, 200);
  evas_object_color_set(o, 255, 255, 255, 255);

  sc->bg = o;

  margin = 10;
  width = 200 - 2 * margin;
  height = 4;

  o = evas_object_rectangle_add(evas_object_evas_get (obj));
  evas_object_move(o, margin, (200 - height) / 2);
  evas_object_resize(o, width, height);
  evas_object_color_set(o, 0, 0, 0, 255);

  sc->o = o;

/*   evas_object_event_callback_add (sc->obj, EVAS_CALLBACK_MOUSE_MOVE, _mouse_move, sc); */
/*   evas_object_event_callback_add (sc->obj, EVAS_CALLBACK_MOUSE_DOWN, _mouse_down, sc); */
  evas_object_smart_member_add (sc->bg, obj);

  evas_object_image_alpha_set (sc->bg, 0);
  evas_object_smart_data_set (obj, sc);
}

static void
_smart_del (Evas_Object *obj)
{
  Smart_Cervoise *sc;

  sc = evas_object_smart_data_get (obj);
  if (!sc) return;

  evas_object_del(sc->o);
  evas_object_del(sc->bg);

  free (sc);
}

static void
_smart_move (Evas_Object *obj, Evas_Coord x, Evas_Coord y)
{
  Smart_Cervoise *sc;

  sc = evas_object_smart_data_get (obj);
  if (!sc) return;

  evas_object_move (sc->bg, x, y);
  evas_object_move (sc->o, x, y);
}

static void
_smart_resize (Evas_Object *obj, Evas_Coord w, Evas_Coord h)
{
  Smart_Cervoise *sc;
  int width;
  int height;
  int margin;

  sc = evas_object_smart_data_get (obj);
  if (!sc) return;

  evas_object_resize (sc->bg, w, h);

  margin = 10;
  width = w - 2 * margin;
  height = 4;

  evas_object_move(sc->o, margin, (h - height) / 2);
  evas_object_resize(sc->o, width, height);
}

static void
_smart_show (Evas_Object *obj)
{
  Smart_Cervoise *sc;

  sc = evas_object_smart_data_get (obj);
  if (!sc) return;

  evas_object_show (sc->bg);
  evas_object_show (sc->o);

}

static void
_smart_hide (Evas_Object *obj)
{
  Smart_Cervoise *sc;

  sc = evas_object_smart_data_get (obj);
  if (!sc) return;

  evas_object_hide (sc->bg);
  evas_object_hide (sc->o);
}

static void
_smart_clip_set (Evas_Object *obj, Evas_Object *clip)
{
  Smart_Cervoise *sc;

  sc = evas_object_smart_data_get (obj);
  if (!sc) return;

  evas_object_clip_set (sc->bg, clip);
  evas_object_clip_set (sc->o, clip);
}

static void
_smart_clip_unset (Evas_Object *obj)
{
  Smart_Cervoise *sc;

  sc = evas_object_smart_data_get (obj);
  if (!sc) return;

  evas_object_clip_unset (sc->bg);
  evas_object_clip_unset (sc->o);
}

